Spree::Order.class_eval do

  self.prepend(Spree::Order::MailDefuser)

  def is_winkelstraat_order?
    winkelstraat_order_id.present?
  end

  class << self

    # {
    #     "id":"401292144",
    #     "date":"2019-04-06T20:35:05+02:00",
    #     "items":[
    #         {
    #             "sku":"32963-40",
    #             "reference":"00002795 AZ",
    #             "article_number":"00002795",
    #             "manufacturer":"Dsquared2",
    #             "name":"Jeans met leg pattern",
    #             "size":"IT 50",
    #             "color":"Blauw",
    #             "quantity":1,
    #             "price":718
    #         }
    #     ],
    #     "discount":0,
    #     "grand_total":718,
    #     "shipping_method":null,
    #     "picking_pdf_url":"https:\/\/www.winkelstraat.nl\/pushapi\/order\/download\/0%3A2%3Aaeed630c14ba637b%3AHK6I26xV2rwoECXttEr5dQ%3D%3D\/2534447b7eef8a7e0878e6195fc7be470d1a07e0b4549e25784bc3816d031c45:yeLKFkZerl1XlrWKNTYR8qz5BNKWEw7v",
    #     "shipping_address":{
    #         "name":"Test de Test",
    #         "street":"J.J.A. Winkelstraat",
    #         "housenumber":"5",
    #         "postcode":"3494RK",
    #         "city":"Ergens",
    #         "province":null,
    #         "country":"NL",
    #         "email":"service@Winkelstraat.nl"
    #     }
    # }
    def winkelstraat_to_order_params(params)
      order_params = {}

      order_params[:winkelstraat_order_id] = params['id']
      order_params[:completed_at] = DateTime.parse(params['date'])
      order_params[:email] = params['shipping_address']['email']
      order_params[:winkelstraat_order_id] = params['id']
      order_params[:use_billing] = true

      order_params[:bill_address_attributes] = build_address(params['shipping_address'])

      order_params[:shipments_attributes] = build_shipments_attributes(params, 'Amstelveen')
      order_params[:payments_attributes] = build_payments_attributes(params)
      order_params[:line_items_attributes] = build_line_items_attributes(params)
      order_params[:adjustments_attributes] = build_adjustments_attributes(params)

      order_params
    end

    # private

    def build_address(address_params)
      {
          firstname: address_params['name'].split(' ').first,
          lastname: address_params['name'].split(' ')[1..-1].join(' '),
          address1: address_params['street'],
          address2: address_params['housenumber'],
          city: address_params['city'],
          zipcode: address_params['postcode'],
          phone: '0612345678',
          country: {
              'iso' => address_params['country']
          }
      }
    end

    #     "shipments_attributes": [
    #       {
    #         "tracking": "track track track",
    #         "stock_location": "default",
    #         "cost": 11,
    #         "shipping_method": "UPS Ground (USD)",
    #         "inventory_units": [
    #           { "variant_id": "1" },
    #           { "variant_id": "2" },
    #           { "variant_id": "2" },
    #           { "variant_id": "2" },
    #           { "variant_id": "2" },
    #           { "variant_id": "3" },
    #           { "variant_id": "4" },
    #           { "variant_id": "5" }
    #         ]
    #       }
    #     ],
    def build_shipments_attributes(params, stock_location_name)
      [
          {
              tracking: nil,
              stock_location: stock_location_name,
              shipping_method: 'Winkelstraat shipping',
              inventory_units: params['items'].map do |item|
                item['quantity'].times.map do
                  {
                      sku: item['sku']
                  }
                end
              end.flatten
          }
      ]
    end

    #     "payments_attributes": [
    #       {
    #         "number": "REGE345546FDF",
    #         "state": "completed",
    #         "amount": 480.90,
    #         "payment_method": "Authorize.Net"
    #       },
    #       {
    #         "number": "ERERGREG43534DF",
    #         "state": "void",
    #         "amount": 46.95,
    #         "payment_method": "Authorize.Net"
    #       }
    #     ],
    def build_payments_attributes(params)
      [
          {
              amount: params['grand_total'],
              state: 'completed',
              payment_method: 'Paid at winkelstraat'
          }
      ]
    end

    #     "line_items_attributes": {
    #       "0": {
    #         "variant_id": "1",
    #         "quantity": 1,
    #         "price": 50
    #       },
    #       "3": {
    #         "variant_id": "2",
    #         "quantity": 4,
    #         "price": 60
    #       },
    #     },
    def build_line_items_attributes(params)
      params['items'].map do |item|
        variant = Spree::Variant.active.find_by_sku(item['sku'])
        next if variant.blank?
        {
            variant_id: variant.id,
            quantity: item['quantity'],
            price: item['price']
        }
      end.compact
    end

    # "adjustments_attributes": [
    #       {
    #         "label": "tax",
    #         "amount": 0
    #       },
    #       {
    #         "label": "something",
    #         "amount": 17.9
    #       }
    #     ],
    def build_adjustments_attributes(params)
      [
          {
              label: 'Winkelstraat Discount',
              amount: -(params['discount'].to_f.abs)
          }
      ]
    end
  end
end