Spree::Product.class_eval do

  WOMEN_CATEGORIES = ["Accessoires", "Badmode", "Blazers & Gilets", "Blouses", "Broeken", "Jassen", "Jeans",
                      "Jumpsuits", "Jurken", "Lingerie", "Nachtmode", "Rokken", "Schoenen", "Sport", "Tassen", "Tops", "Truien & Vesten"]
  MEN_CATEGORIES = ["Accessoires", "Badmode", "Broeken", "Colberts & Gilets", "Jassen", "Jeans", "Ondergoed",
                    "Overhemden", "Schoenen", "Shirts", "Sport", "Truien", "Vesten"]
  KIDS_CATEGORIES = ["Jongens", "Meisjes"]

  IMG_HOST = Rails.env.production? ? 'https://paulwarmer-production.imgix.net' : Settings.backend_url

  class << self
    def winkelstraat_csv
      products = Spree::Product.active.where(sell_on_winkelstraat: true)
      variants = Spree::Variant.includes(:images, option_values: :translations, product: :translations).where(product: products, is_master: false)

      headings = [
          'Productnaam',
          'Winkel product nummer',
          'EAN',
          'Merk product nummer',
          'Kassa referentie',
          'Geslacht',
          'Categorie',
          'Subcategorie',
          'Merk',
          'Kleur',
          'Prijs',
          'Maatvoering',
          'Materiaal',
          'Pasvorm',
          'Omschrijving',
          'Maat op foto',
          'Lengte',
          'Breedte',
          'Hoogte',
          'Omvang',
          'Seizoen',
          'Carry over',
          *(1..10).to_a.map {|i| "Afbeelding#{i}"},
          *Spree::StockLocation.all.map {|sl| sl.name }
      ]

      women_taxon = Spree::Taxon.where(name: 'women').first
      men_taxon = Spree::Taxon.where(name: 'men').first
      kids_taxon = Spree::Taxon.where(name: 'kids').first

      main_taxonomy = Spree::Taxonomy.where(name: 'Winkelstraat Category').first
      brand_taxonomy = Spree::Taxonomy.where(name: 'Filter').first
      brand_taxon_parent = Spree::Taxon.where(taxonomy: brand_taxonomy, name: 'Brands').try(:first)

      season_taxonomy = Spree::Taxonomy.where(name: 'Season').first


      CSV.generate(headers: true) do |csv|
        csv << headings

        variants.each do |variant|

          gender = case
                   when variant.in_taxon?(women_taxon)
                     'D'
                   when variant.in_taxon?(men_taxon)
                     'H'
                   when variant.in_taxon?(kids_taxon)
                     'K'
                   else
                     'D'
                   end

          csv << [
              variant.product.ws_compatible_name, #'Productnaam',
              variant.id, #'Winkel product nummer',
              variant.sku, # 'EAN',
              variant.sku, # 'Merk product nummer',
              variant.kassa_reference, # 'Kassa referentie',
              gender, #Geslacht
              variant.product.taxons.where(taxonomy: main_taxonomy, parent: main_taxonomy.root).first.try(:name), # 'Categorie',
              variant.product.taxons.where(taxonomy: main_taxonomy).where.not(parent: nil).where.not(parent: main_taxonomy.root).first.try(:name), #'Subcategorie',
              brand_taxon_parent.present? ? variant.product.taxons.where(parent: brand_taxon_parent).first.try(:name) : nil, #'Merk',
              variant.option_values.joins(option_type: :translations).where(spree_option_type_translations: { name: 'color' }).first.try(:ws_code), #'Kleur',
              variant.price, # 'Prijs',
              'CF', #Maatvoering',
              variant.product.product_properties.includes(property: :translations).where(spree_property_translations: {name: 'material'}).first.try(:value), #Materiaal',
              variant.product.product_properties.includes(property: :translations).where(spree_property_translations: {name: 'fit'}).first.try(:value) || 'Dit item valt normaal', #Pasvorm',
              ActionController::Base.helpers.strip_tags(variant.description), #'Omschrijving',
              variant.product.variants.first.option_values.joins(option_type: :translations).where(spree_option_type_translations: { name: 'size' }).first.try(:presentation), #'Maat op foto',
              variant.depth, #lengte
              variant.width, #breedte
              variant.height, #hoogte
              nil, #'Omvang',
              variant.product.taxons.where(taxonomy: season_taxonomy).first.try(:name), # 'Seizoen',Afbeeldingen
              nil, #'Carry over',
              *(array = (variant.images.any? ? variant.images : variant.product.master.images).first(10).map do |image|
                uri = URI.join(IMG_HOST, image.attachment(:original))
                uri.query = URI.encode_www_form [['fm', 'jpg'], ['w', '1280'], ['h', '1920'], ['fit', 'clamp']]
                uri.to_s
              end
              array.fill(nil, array.size..9)),
              *Spree::StockLocation.all.map {|sl| sl.count_on_hand(variant) || 0 }
          ]
        end
      end

    end
  end

  def ws_compatible_name
    name
  end

end
